#include <Arduino.h>
#include "I2COscillator.h"

I2COscillator::I2COscillator(I2C * _i2c, int _i2caddr, float _freq, byte _bitScale) {
    theI2C = _i2c;
    i2cAddress = _i2caddr;
    requestedFreq = _freq;
    bitScale = _bitScale;
}

bool I2COscillator::begin() {     
    float foct;
    unsigned int ioct;
    float fdac;
    unsigned int idac;
    float scaleFact = 1 << bitScale;

    float fFreq = requestedFreq * scaleFact;
    if (fFreq < 1039.0) {
        fFreq = 1039.0;
    } else if (fFreq > 68030000.0) {
        fFreq = 68030000.0;
    }

    foct = 3.322 * log10(fFreq / 1039.0);
    ioct = trunc(foct);

    fdac = 2048.0 - ((2078.0 * pow(2, (ioct + 10))) / fFreq);
    idac = round(fdac);

    actualFreq = (pow(2, ioct) * (2078.0 / (2.0 - (idac / 1024.0))));
    actualFreq = actualFreq / scaleFact;
    
    byte dat0;
    byte dat1;

    dat1 = ((ioct << 4) & 0xf0) | ((idac >> 6) & 0x0f);
    dat0 = ((idac << 2) & 0xfc) | 0x02;

    theI2C->beginTransmission(i2cAddress);
    theI2C->write(dat1);
    theI2C->write(dat0);
    theI2C->endTransmission();
    
    return true;
}

float I2COscillator::getActualFreq() {
    return actualFreq;
}

float I2COscillator::getRequestedFreq() {
    return requestedFreq;
}

float I2COscillator::getMillisPerTick() {
    return 1000.0 / actualFreq;
}

float I2COscillator::getMicrosPerTick() {
    return 1000000.0 / actualFreq;
}
