#include "InterruptManager.h"

InterruptManager * theManager = InterruptManager::getManager();
InterruptManager interruptManager = *theManager;

typedef void (* interruptFunctionType) (void);

#define IRFUNC(n) void InterruptFunction_##n() { theManager->handleInterrupt(n); }
#include "InterruptFuncDefs.h"

#define IRFUNC(n) InterruptFunction_##n,
interruptFunctionType interruptFunctions[] =
{
#include "InterruptFuncDefs.h"
};
#define IRFUNC(n) digitalPinToInterrupt(n),
byte irNoForPin[] =
{
#include "InterruptFuncDefs.h"    
};

InterruptManager * InterruptManager::getManager() {
    if (theManager == NULL) {
        theManager = new InterruptManager();
    }
    return theManager;
}

void InterruptManager::removeManager() {
    if (theManager != NULL) {
        delete theManager;
    }
    theManager = NULL;
}

InterruptManager::InterruptManager() {
    handlerMap = new Map<byte, AbstractInterruptHandler *>();
}

byte InterruptManager::getPinForIrNo(byte irNo) {
    int i;
    
    for (i = 0; i < NUM_DIGITAL_PINS; i++) {
        if (irNoForPin[i] == irNo) {
            return i;
        }
    }
    
    return NOT_AN_INTERRUPT;
}

bool InterruptManager::attachHandlerToPin(AbstractInterruptHandler * handler, byte pin, byte mode, bool pullup) {
    int irNo = digitalPinToInterrupt(pin);
    if (irNo == NOT_AN_INTERRUPT) {
        return false;
    }
    AbstractInterruptHandler * currHandler;
    if (handlerMap->get(pin, currHandler)) {
        detachHandlerFromPin(pin);
    }
    pinMode(pin, pullup ? INPUT_PULLUP : INPUT);
    handlerMap->add(pin, handler);
    attachInterrupt(irNo, interruptFunctions[pin], mode);
    handler->onAttach(pin);
    return true;
}

bool InterruptManager::attachHandlerToInterrupt(AbstractInterruptHandler * handler, byte irNo, byte mode, bool pullup) {
    byte pin = getPinForIrNo(irNo);
    if (pin == NOT_AN_INTERRUPT) {
        return false;
    }
    return attachHandlerToPin(handler, pin, mode, pullup);
}

void InterruptManager::detachHandlerFromPin(byte pin) {
    int irNo = digitalPinToInterrupt(pin);
    if (irNo == NOT_AN_INTERRUPT) {
        return;
    }
    detachInterrupt(irNo);
    AbstractInterruptHandler * handler;
    if (handlerMap->get(pin, handler)) {
        handler->onDetach(pin);
    }
    handlerMap->remove(pin);
}

void InterruptManager::detachHandlerFromInterrupt(byte irNo) {
    byte pin = getPinForIrNo(irNo);
    if (pin == NOT_AN_INTERRUPT) {
        return;
    }
    detachHandlerFromPin(pin);
}

void InterruptManager::handleInterrupt(byte pin) {
    AbstractInterruptHandler * handler;
    if (handlerMap->get(pin, handler)) {
       if (handler->isEnabled() && handler->wantInterrupt(pin)) {
          handler->handleInterrupt(pin);
       }
    }
}
